###
#
# Listen is the legal property of mehdi abaakouk <theli48@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###

import gtk
import threading
from time import time


from view_song import *
from db_manager import *
from misc_widget import get_scrolled_window


REFRESH_TIME = 250000

def make_label(text):
    label = gtk.Label()
    label.set_use_markup(True)
    label.set_markup("<span size=\"large\"><b>"+text+"</b></span>")
    label.set_alignment(0,0.5)
    label.set_size_request(-1,22)
    box = gtk.HBox(False,6)
    box.pack_start(gtk.Arrow(gtk.ARROW_RIGHT,gtk.SHADOW_NONE),False,False)
    box.pack_start(label,True,True)
    return box

class ListenHome(gtk.VBox):
    def __init__(self,media_organizer):
        gtk.VBox.__init__(self,False,6)
        self.set_border_width(6)
        self.media_organizer = media_organizer


        #LASTEST
        self.lastest_view = ListenDetailSongView(media_organizer)#,"last_played"

        #PREFERED
        self.prefered_song_view = ListenDetailSongView(media_organizer,"playcount")

        #albums
        self.albums_view = ListenAlbumView(media_organizer,True)

        btn = gtk.Button()
        btn.connect("clicked",self.new_media_cb)
        self.make_hbox_item(make_label(_("Last played songs")),make_label(_("Preferred songs")),False)
        self.make_hbox_item(get_scrolled_window(self.lastest_view),get_scrolled_window(self.prefered_song_view),True)
        self.pack_start(make_label(_("Preferred albums")),False,False)
        self.pack_start(get_scrolled_window(self.albums_view),True,True)

    def run(self):
        gobject.idle_add(self.new_media_cb,False)
        gobject.timeout_add(REFRESH_TIME,self.new_media_cb,True)

    def make_hbox_item(self,item1,item2,extend=True):
        box = gtk.HBox(True,6)
        box.pack_start(item1,True,True)
        box.pack_start(item2,True,True)
        self.pack_start(box,extend,extend)

    def new_media_cb(self,return_value):
        #Write the file to open a new sqlite instance
        #in the thread with info update the date
        DBManager.write()
        thread = threading.Thread(target=self.fill_info_played)
        thread.start()
        thread = threading.Thread(target=self.fill_info_prefered)
        thread.start()
        thread = threading.Thread(target=self.fill_info_album)
        thread.start()
        return return_value

    def fill_info_played(self):
        DB = ListenDBManager()

        #Last playeds
        #SQLITE HACK CAST(last_played as int) == last_played+1-1
        requete = "SELECT s.* FROM songs s WHERE last_played is not null AND last_played <> '0' ORDER BY last_played+1-1 DESC LIMIT 20"
        songs = DB.get_songs_custom(requete)
        gobject.idle_add(self.lastest_view.model_song.clear)

        for song in songs:
            gobject.idle_add(self.add_lastest_song,song)

    def fill_info_prefered(self):
        DB = ListenDBManager()
        #Prefered Songs
        #SQLITE HACK CAST(playcount as int) == playcount+1-1
        requete = "SELECT s.* FROM songs s where playcount is not null AND playcount<>'0' ORDER BY playcount+1-1 DESC LIMIT 20"
        songs = DB.get_songs_custom(requete)
        gobject.idle_add(self.prefered_song_view.model_song.clear)

        for song in songs:
            gobject.idle_add(self.add_prefered_song,song)

        #Prefered Artists

    def fill_info_album(self):
        DB = ListenDBManager()
        #Prefered Albums
        #SQLITE HACK CAST(playcount as int) == playcount+1-1
        albums = DB.get_albums(" playcount is not null AND playcount<>'0' ","playcount+1-1 DESC LIMIT 10")

        gobject.idle_add(self.albums_view.model_song.clear)
        for album in albums:
            #make a fake song to get album cover
            #Must change to get a real song from this album
            #it's better for search cover (more attribut like uri)
            song = Song()
            song.set_property("artist",album["artist"])
            song.set_property("album",album["album"])
            filename = song.get_album_art()
            album["image"] = gtk.gdk.pixbuf_new_from_file_at_size(filename,40,40)
            gobject.idle_add(self.add_album,album)

    def add_album(self,album):
        self.albums_view.model_song.append((album,))

    def add_lastest_song(self,song):
        self.lastest_view.model_song.append((song,song.get_int("tracknr"),song.sprint("title"),song.get_property("duration"),song.sprint("artist"),song.sprint("album")))

    def add_prefered_song(self,song):
        self.prefered_song_view.model_song.append((song,song.get_int("tracknr"),song.sprint("title"),song.get_property("duration"),song.sprint("artist"),song.sprint("album")))

class ListenInformation(gtk.VBox):
    def __init__(self,media_organizer):
        gtk.VBox.__init__(self,False,6)
        self.set_border_width(6)
        self.media_organizer = media_organizer
        self.player = media_organizer.player

        self.player.gst_player.connect("new-media",self.new_media_cb)

        self.song = None
        self.albums_label = gtk.Label()
        self.albums_label.set_use_markup(True)
        self.albums_label.set_markup("<span size=\"large\"><b>"+_("Albums")+"</b></span>")
        self.albums_label.set_alignment(0,0.5)
        self.albums_label.set_size_request(-1,22)
        self.albums_view = ListenAlbumView(self.player)

        self.prefered_label = gtk.Label()
        self.prefered_label.set_use_markup(True)
        self.prefered_label.set_markup("<span size=\"large\"><b>"+_("Preferred")+"</b></span>")
        self.prefered_label.set_alignment(0,0.5)
        self.prefered_label.set_size_request(-1,22)
        self.prefered_view = ListenDetailSongView(self.media_organizer,"playcount",False)

        box = gtk.HBox(False,6)
        box.pack_start(gtk.Arrow(gtk.ARROW_RIGHT,gtk.SHADOW_NONE),False,False)
        box.pack_start(self.albums_label,True,True)
        self.pack_start(box,False,False)
        self.pack_start(get_scrolled_window(self.albums_view),True,True)

        box = gtk.HBox(False,6)
        box.pack_start(gtk.Arrow(gtk.ARROW_RIGHT,gtk.SHADOW_NONE),False,False)
        box.pack_start(self.prefered_label,True,True)
        self.pack_start(box,False,False)
        self.pack_start(get_scrolled_window(self.prefered_view),True,True)


    def new_media_cb(self,widget,song):
        self.song = song

        if self.song==None:
            return
        #Write the file to open a new sqlite instance
        #in the thread with info update the date
        DBManager.write()
        thread = threading.Thread(target=self.fill_info_album)
        thread.start()

        thread = threading.Thread(target=self.fill_info_song)
        thread.start()

    def fill_info_album(self):

        DB = ListenDBManager()

        #Albums by
        albums = DB.get_albums("artist='"+utils.escape(self.song.sprint("artist"))+"'")

        gobject.idle_add(self.albums_label.set_markup,"<span size=\"large\"><b>"+_("Albums by") +" "+ self.song.sprint("artist")+"</b></span>")
        gobject.idle_add(self.albums_view.model_song.clear)
        for album in albums:

            #make a fake song to get album cover
            song = Song()
            song.set_property("artist",album["artist"])
            song.set_property("album",album["album"])
            filename = song.get_album_art()
            album["image"] = gtk.gdk.pixbuf_new_from_file_at_size(filename,40,40)
            gobject.idle_add(self.add_album,album)

    def fill_info_song(self):
        DB = ListenDBManager()
        #Prefered by
        SELECT = "SELECT s.* FROM songs s "
        WHERE = "WHERE artist='"+utils.escape(self.song.sprint("artist"))+"' "
        WHERE += " AND (playcount is not null AND playcount<>'0')"
        #SQLITE HACK CAST(playcount as int) == playcount+1-1
        ORDERBY = "ORDER BY playcount+1-1 DESC LIMIT 20"
        requete = SELECT+WHERE+ORDERBY
        songs = DB.get_songs_custom(requete)

        gobject.idle_add(self.prefered_view.model_song.clear)
        gobject.idle_add(self.prefered_label.set_markup,"<span size=\"large\"><b>"+_("Preferred songs by") +" "+ self.song.sprint("artist")+"</b></span>")
        for song in songs:
            gobject.idle_add(self.add_prefered_song,song)

    def add_album(self,album):
        self.albums_view.model_song.append((album,))

    def add_prefered_song(self,song):
        self.prefered_view.model_song.append((song,song.get_int("tracknr"),song.sprint("title"),song.get_property("duration"),song.sprint("artist"),song.sprint("album")))


class ListenDetailSongView(ListenUITreeviewSong):
    def __init__(self,media_organizer,info_to_display=None,show_album=True):
        #use the song for menu and drag drop but use only title column for display infomation
        allow_column={"title":True,"album":False,"artist":False,"track":False,"duration":False}
        ListenUITreeviewSong.__init__(self,media_organizer,allow_column,False)

        self.show_album = show_album
        self.info_to_display = info_to_display

        self.set_model(self.model_song)
        self.set_headers_visible(False)
        self.get_column(0).set_cell_data_func(self.renderer_song, self.cell_data_func)


    def cell_data_func(self, column, cell, model, iter):
        song = model.get_value(iter, 0)
        text = "<span><b>"+xml.sax.saxutils.escape(song.sprint("title"))+"</b></span><span size=\"small\">"
        """if self.info_to_display=="playcount":
            text +=  " <i>"+_("played")+" "+song.sprint("playcount")+" "+_("times")+"</i>"
        elif self.info_to_display=="last_played":
            if song.get_property("last_played")!=None:
                text += " <i>"+_("played")+" "+utils.duration_to_string(int(time())-int(song.get_property("last_played")))+" "+_("ago")+"</i>"
        """
        text += "\n<i>"+xml.sax.saxutils.escape(song.sprint("artist"))+"</i>"
        if self.show_album:
            text += " - "+xml.sax.saxutils.escape(song.sprint("album"))
        text += "</span>"
        if config.get("setting","mlib_smallfont")=="true":
            text="<span size=\"small\">"+text+"</span>"

        cell.set_property("markup",text )


class ListenAlbumView(gtk.TreeView):
    def __init__(self,media_organizer,show_artist=False):
        gtk.TreeView.__init__(self)

        self.media_organizer = media_organizer
        self.show_artist = show_artist
        self.model_song = gtk.ListStore(object)

        self.renderer_cover = gtk.CellRendererPixbuf()
        self.renderer_cover.set_property("height",50)
        self.renderer_cover.set_property("width",50)
        self.renderer_album = gtk.CellRendererText()
        self.renderer_album.set_property("height",50)
        self.renderer_album.set_property("ellipsize",pango.ELLIPSIZE_END)
        #self.renderer_album.set_property("yalign",0)

        targets = [("text/uri-list", 0, 0)]
        self.enable_model_drag_source(
            gtk.gdk.BUTTON1_MASK|gtk.gdk.SHIFT_MASK|gtk.gdk.CONTROL_MASK, targets,
            gtk.gdk.ACTION_COPY)

        self.column_cover = gtk.TreeViewColumn()
        self.column_cover.pack_start(self.renderer_cover,True)
        self.column_cover.set_cell_data_func(self.renderer_cover, self.__cell_data_func_cover)

        self.column_album = gtk.TreeViewColumn()
        self.column_album.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        self.column_album.pack_start(self.renderer_album,True)
        self.column_album.set_cell_data_func(self.renderer_album, self.__cell_data_func_album)


        self.append_column(self.column_cover)
        self.append_column(self.column_album)
        self.set_headers_visible(False)
        self.set_rules_hint(True)
        self.connect("row-activated", self.on_row_song_activated)
        self.connect("drag-data-get", self.on_song_drag_data_get)

        self.set_model(self.model_song)


    def on_row_song_activated(self, treeview, path, view_column):
        model, rows = self.get_selection().get_selected_rows()
        play = True
        for row in rows:
            album = model.get_value(model.get_iter(row[0]),0)
            songs = DBManager.get_all_songs("artist='"+utils.escape(album["artist"])+"' and "+\
                                        "album='"+utils.escape(album["album"])+"'")

            for song in songs:
                self.media_organizer.player.playlist.add_song(song,False,play)
                play = False


    def on_song_drag_data_get(self,treeview, context, selection, info, timestamp):

        model, rows = self.get_selection().get_selected_rows()

        list_uri = []
        list_song = []
        for row in rows:
            album = model.get_value(model.get_iter(row[0]),0)
            songs = DBManager.get_all_songs("artist='"+utils.escape(album["artist"])+"' and "+\
                                        "album='"+utils.escape(album["album"])+"'")
            for song in songs:
                list_song.append(song)
                uri = song.get_property("uri")
                uri = utils.convert_to_uri(uri)
                list_uri.append(uri)
                #print uri

        self.get_toplevel().last_drag_drop = list_song
        selection.set_uris(list_uri)


    def __cell_data_func_cover(self, column, cell, model, iter):
        album = model.get_value(iter, 0)
        cell.set_property("pixbuf",album["image"] )


    def __cell_data_func_album(self, column, cell, model, iter):
        album = model.get_value(iter, 0)
        if album["album"]=="":
            title = _("Unknown")
        else:
            title = album["album"]

        if album["nb_song"]<=1:
            value = _("song")
        else:
            value = _("songs")

        duration  = " - "+utils.duration_to_string(album["duration"])

        if self.show_artist:
            if album["artist"]=="":
                title += " - <i>"+_("Unknown")+"</i>"
            else:
                title += " - <i>"+album["artist"]+"</i>"

        #+" - %d"%int(album["playcount"])
        value = "<b>"+title + "</b>\n%d"%album["nb_song"]+" "+value+duration

        if config.get("setting","mlib_smallfont")=="true":
            value="<span size=\"small\">"+value+"</span>"
        #print "TITLE: ",title," - VALEUR: ",value
        cell.set_property("markup",value )



