###
#
# Listen is the legal property of mehdi abaakouk <theli48@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###

import os, sys
import gobject


import utils
from hal import hal
from song import *

DEBUG={}
DEBUG["requete"]=False

from pysqlite2 import dbapi2 as sqlite

class ListenDBManager(object):
    def __init__(self):
        self.db = ListenDB()

    def set_song_property(self,id,property,value):
        self.db.set_media_property(id,property,value)

    def get_all_songs(self,where="",orderby=""):
        if where!="":
            where = "WHERE "+where

        if orderby=="":
            #SQLITE Hack
            #tracknr+1-1 == CAST (tracknr as int)
            # this use auto type convertion feature of sqlite
            #because the CAST method appear only since the version 3.2.3

            orderby="LOWER(artist),LOWER(album),tracknr+1-1,LOWER(title)"
        query = "SELECT * FROM songs "+where+" ORDER BY "+orderby+";"
        res = self.db.select_request(query)

        array_song = []
        for record in res:
            song = Song()
            song.read_from_dict(record)
            array_song.append(song)

        return array_song
	
    def get_random_song(self):
	query = "SELECT * FROM songs WHERE id>=abs(random(*))%(SELECT max(id) from songs) LIMIT 1;"
	res = self.db.select_request(query)
	
	if len(res)== 0 or res==None:
	   song = None
	else:
	   song = Song()
	   song.read_from_dict(res[0])
	return song
	
    def get_songs_custom(self,query):

        res = self.db.select_request(query)
        array_song = []
        for record in res:
            song = Song()
            song.read_from_dict(record)
            array_song.append(song)

        return array_song

    def close(self):
        self.write()

    def write(self):
        self.db.write()

    def update_song(self,song,list_property=None):
        #FIXME: Replace dict by song
        dict = song.get_dict()
        for property,value in dict.iteritems():
            if list_property == None or property in list_property:
                if property in song.integer_key and value!=None:
                    value = "%d"%value
                    self.set_song_property(song.get_property("id"),property,value)
                elif value!=None:
                    if isinstance(value,bool):
                        if value: value="1"
                        else: value="0"
                    value=utils.escape(value)
                    self.set_song_property(song.get_property("id"),property,value)



    def get_iradio(self):
        query = "SELECT * from iradio;"
        res = self.db.select_request(query)
        array_song = []
        for record in res:
            song = Song()
            song.read_from_dict(record)
            song.is_stream=True
            array_song.append(song)

        return array_song


    def delete_song(self,song):
        self.db.delete_media(song.get_property("id"))

    def get_song_by_id(self,id,try_iradio=False):#iradio for playlist
        query = "SELECT * FROM songs WHERE id=%d;"%id
        res = self.db.select_request(query)
        if len(res)==0 or res==None:
            query = "SELECT * FROM iradio WHERE id=%d;"%id
            res = self.db.select_request(query)
            if len(res)==0 or res==None:
                return None
            else:
                song = Song()
                song.is_stream=True
                song.read_from_dict(res[0])
        else:
            song = Song()
            song.read_from_dict(res[0])
        return song

    def get_song_by_uri(self,uri,parse_not_exist=True):
        if not os.path.exists(utils.get_path(uri)):
            return None
        query = "SELECT * FROM songs WHERE uri='%s';"%utils.escape(uri)
        res = self.db.select_request(query)
        song = Song()
        if len(res)==0 or res==None:
            id = self.db.get_or_add_media(uri)
            song.set_property("uri",utils.fsdecode(uri))
            song.set_property("id",id)
            #Add control if not a local file
            if parse_not_exist:
                if song.read_from_file():
                    self.update_song(song)
                else:
                    self.delete_song(song)
                    #print utils.fsdecode(uri),": ", song.last_error
                    return None
        else:
            song.read_from_dict(res[0])
        return song

    def get_iradio_by_uri(self,uri):

        query = "SELECT * FROM iradio WHERE uri='%s';"%utils.escape(uri)
        res = self.db.select_request(query)
        song = Song()
        if len(res)==0 or res==None:
            id = self.db.get_or_add_media(uri)
            song.set_property("uri",utils.fsdecode(uri))
            song.set_property("id",id)
            song.set_property("iradio","1")
            self.update_song(song)
        else:
            song.read_from_dict(res[0])
        return song


    def get_songs_from_playlist(self,id_playlist):
        res = self.db.select_request("SELECT id_media FROM playlists WHERE id_playlist=%d ORDER by pos;"%id_playlist)
        list_song = []
        for i in range(0,len(res)):
            song = self.get_song_by_id(res[i]["id_media"],(id_playlist==0))
            if song!=None:
                list_song.append(song)
        return list_song

    def get_songs_from_playlist_by_name(self,name_playlist):
        return self.get_songs_from_playlist(self.get_playlist_by_name(name_playlist)["id"])

    def get_playlists(self,where=""):
        if where!="":
            where = "WHERE "+where
        return self.db.select_request("SELECT * FROM playlist "+where+" ORDER BY name")

    def get_playlist_by_name(self,name_playlist):
        res = self.db.select_request("SELECT * FROM playlist WHERE name='%s' LIMIT 1;"%utils.escape(name_playlist))
        if len(res)==1:
            return res[0]
        else:
            return None

    def add_playlist(self,name):
        query = "INSERT INTO playlist (name) VALUES ('%s');"%utils.escape(name)
        self.db.execute(query)
        return self.get_playlist_by_name(name)["id"]


    def add_song_to_playlist(self,song,id):
        query = """
            INSERT INTO playlists (id_playlist,id_media,pos)
            VALUES (
                    %d,%d,
                    (SELECT max(pos) FROM playlists WHERE id_playlist=%d)
                   );
        """%(id,song.get_property("id"),id)
        self.db.execute(query)

    def clear_playlist(self,id):
        self.db.execute("DELETE FROM playlists WHERE id_playlist=%d;"%id)

    def delete_playlist(self,id):
        self.clear_playlist(id)
        self.db.execute("DELETE FROM playlist WHERE id=%d;"%id)

    def rename_playlist(self,id,name):
        self.db.execute("UPDATE playlist SET name='%s' WHERE id=%d ;"%(utils.escape(name),id))

    def get_albums(self,where="",orderby="LOWER(album), LOWER(artist)"):
        if where!="":
            where = "WHERE "+where
        query = "SELECT * FROM albums "+where+" ORDER by "+orderby+" ;"
        res = self.db.select_request(query)
        return res

    def get_artists(self,where="",orderby="LOWER(artist)"):
        if where!="":
            where = "WHERE "+where
        query = "SELECT * FROM artists "+where+" ORDER by "+orderby+" ;"
        res = self.db.select_request(query)
        return res

    def select_request(self,query):
        return self.db.select_request(query)



class ListenDB( gobject.GObject ):
    def __init__( self ):
        gobject.GObject.__init__( self )
        self.verbose = False
        gobject.threads_init()
        self.HOME_DIR    = os.path.expanduser( "~" )
        self.CONFIG_DIR     = os.path.join( self.HOME_DIR+"/.listen/" )
        self.DB_NAME  = os.path.join( self.CONFIG_DIR, "media.db" )

        if not os.path.isfile( self.DB_NAME ):
            self.con = sqlite.connect( self.DB_NAME )
            self.cur = self.con.cursor()
            print "LISTEN : CREATE DATABASE"
            requete_media = """
               CREATE TABLE
                    Media
                    (
                      id       integer,
                      key      varchar(20),
                      value    BLOB,
                      PRIMARY KEY (id,key)
                    );
                """
            requete_songs_view = """
                CREATE VIEW songs AS
                        SELECT m1.id as id , m1.value as uri, m2.value as title, m3.value as album,
                                m4.value as artist, m5.value as duration, m6.value as tracknr,m10.value as playcount
                                ,m11.value as last_played
                        FROM Media m1
                             LEFT OUTER JOIN Media m2 ON m1.id = m2.id AND m2.key='title'
                             LEFT OUTER JOIN Media m3 ON m1.id = m3.id AND m3.key='album'
                             LEFT OUTER JOIN Media m4 ON m1.id = m4.id AND m4.key='artist'
                             LEFT OUTER JOIN Media m5 ON m1.id = m5.id AND m5.key='duration'
                             LEFT OUTER JOIN Media m6 ON m1.id = m6.id AND m6.key='tracknr'
                             LEFT OUTER JOIN Media m7 ON m1.id = m7.id AND m7.key='iradio'
                             LEFT OUTER JOIN Media m10 ON m1.id = m10.id AND m10.key='playcount'
                             LEFT OUTER JOIN Media m11 ON m1.id = m11.id AND m11.key='last_played'
                             LEFT OUTER JOIN Media m8 ON m1.id = m8.id AND m8.key='removable'
                        WHERE m1.key = 'uri' AND (m7.value='0' OR m7.value is null)
                                AND (m8.value='0' OR m8.value is null)
                    ;

                """
            #SQLITE HACK duration+1-1 == CAST (duration as int)
            requete_albums_view = """
                CREATE VIEW albums AS
                    SELECT album,artist ,count(id) as nb_song,
                    avg(playcount) as playcount ,
                    sum(duration+1-1) as duration
                    FROM songs
                    GROUP BY album,artist
                ;

                """
            #SQLITE HACK duration+1-1 == CAST (duration as int)
            requete_artists_view = """
                CREATE VIEW artists AS
                    SELECT artist,count(album) as nb_album ,sum(nb_song)  as nb_song,
                    avg(playcount) as playcount,
                    sum(duration+1-1) as duration
                    FROM (SELECT artist ,album,count(id) as nb_song,
                        avg(playcount) as playcount,
                        sum(duration+1-1) as duration
                        FROM songs GROUP BY artist,album)
                    GROUP BY artist
                ;
                """

            #More simple but don't work in sqlite < 2.3.6
            __requete_artists_view = """
                CREATE VIEW artists AS
                    SELECT artist,count(DISTINCT (album)) as nb_album ,count(id)  as nb_song,
                    avg(playcount) as playcount,
                    sum(duration+1-1) as duration
                    FROM songs
                    GROUP BY artist
                ;
                """

            requete_iradio_view = """
                CREATE VIEW iradio AS
                    SELECT m1.id as id , m1.value as uri, m2.value as title,m7.value as iradio
                    FROM Media m1
                         LEFT OUTER JOIN Media m2 ON m1.id = m2.id AND m2.key='title'
                         LEFT OUTER JOIN Media m7 ON m1.id = m7.id AND m7.key='iradio'
                    WHERE m1.key = 'uri' AND m7.value
                    ;
                """
            requete_resolved_view = """
                CREATE VIEW resolved AS
                SELECT m1.id as id, m1.value as resolved, m2.value as uri
                FROM Media m1 ,Media m2
                WHERE m1.id = m2.id AND
                    m1.key='resolved' AND
                    m1.value = 0 AND m2.key='uri';
                """
            requete_playlist_view = """
                CREATE TABLE playlist
                    (
                      id       integer PRIMARY KEY AUTOINCREMENT,
                      name      varchar(20)
                    );

                """
            requete_playlists_view = """
                CREATE TABLE playlists
                    (
                      id       integer PRIMARY KEY AUTOINCREMENT,
                      id_playlist       integer,
                      id_media      integer,
                      pos        integer UNIQUE
                    );
                """

            requete_playlist_view_first_value = """INSERT INTO playlist VALUES (0,"__autosaved__");"""
            self.cur.execute( requete_media )
            self.cur.execute( requete_songs_view )
            self.cur.execute( requete_albums_view )
            self.cur.execute( requete_artists_view )
            self.cur.execute( requete_resolved_view )
            self.cur.execute( requete_playlist_view )
            self.cur.execute( requete_playlist_view_first_value )
            self.cur.execute( requete_playlists_view )
            self.cur.execute( requete_iradio_view )

            self.con.commit()
            self.ipod_mount_point = {}
        else:
            self.con = sqlite.connect( self.DB_NAME )
            self.cur = self.con.cursor()


    def execute( self, request ):
        self.cur.execute( request )


    def select( self, request ):
        self.debug( "--> select(%s)"%request )
        self.cur.execute( request )
        res = []
        for row in self.cur:
            res.append( row )
        self.debug( "<-- select(%s)"%request )
        return res

    def select_request( self, requete, debug=False ):
        if DEBUG["requete"]: print requete
        self.cur.execute( requete )
        #try:
        #    self.cur.execute( requete )
        #except :raise "SQL ERROR :",requete
        field_list = []
        res = []
        if self.cur.description is None:
            return res;

        for fieldDesc in self.cur.description:
            field_list.append( fieldDesc[0] )
        fieldIndices = range( len( self.cur.description ) )

        for row in self.cur:
            if self.verbose : print "row :", row
            dict = {}
            for fieldIndex in fieldIndices:

                if isinstance(row[fieldIndex],int) or isinstance(row[fieldIndex],float):
                    dict[field_list[fieldIndex]] = int(row[fieldIndex])
                elif  row[fieldIndex]  == None:
                    dict[field_list[fieldIndex]] = ""
                else:
                    dict[field_list[fieldIndex]] = utils.unescape(row[fieldIndex])
            res.append( dict )
        return res

    def delete_media_property( self, id, key ):
        self.debug( "--> delete_media_property(%d,%s)"%( id, key ) )
        self.cur.execute( "DELETE FROM Media WHERE id=%d and key='%s';"%( id, key ) )
        self.debug( "<-- delete_media_property(%d,%s)"%( id, key ) )

    def set_media_property( self, id, key, value, integer=False ):

        self.debug( "--> set_media_property(%d,%s,%s)"%( id, key, value ) )

        self.delete_media_property( id, key )
        query = "INSERT INTO Media (id, key,value) VALUES (%d,'%s','%s') ;"%( id, key, value )
           #query = u"INSERT INTO Media (id, key,value) VALUES (%d"%id+",'"+key+"','"+value+"') ;"
        self.debug(query)
        try:
            self.cur.execute( query )
        except :
            print query
            #print self.get_media(id)
            self.close()
            raise
            exit()
        self.debug("<-- set_media_property(%d,%s,%s)"%(id,key,value))


    def delete_media(self,id):
        self.debug("--> delete_media(%d)"%id)
        self.cur.execute("DELETE FROM Media WHERE id=%d ;"%id)
        self.debug("<-- delete_media(%d)"%id)


    def get_or_add_media(self,uri):

        #Verify Media not already exist
        self.debug("--> add_media("+uri+")")
        uri = utils.escape(uri)
        self.cur.execute("SELECT id FROM Media WHERE key='uri' and value='%s' ;"%uri)
        id = 0
        number_row = 0
        for row in self.cur:
            number_row += 1
            id = row[0]

        if number_row == 0: #Media not exist add it
            self.cur.execute("SELECT max(id) as new_id FROM Media;")
            new_id = 0
            for row in self.cur:
                #print row
                if row[0] == None:
                    new_id = 0
                else:
                    new_id = row[0]
                    new_id +=1

            self.debug("NOT EXIST : NEW ID = %d"%new_id)
            self.cur.execute("INSERT INTO Media (id, key,value) VALUES (%d,'uri','%s') ;"%(new_id,uri))
            return new_id

        elif number_row == 1:
            self.debug("ALREADY EXIST : ID = %d set unresolved for update"%id)
            return id

        else :
            print "ERROR : Media entry %s appear more than 1 times in Media Library"%uri
            sys.exit(-1)

        self.debug("<-- add_media()")
        return None

    def write(self):
        self.con.commit()

    def debug(self,str):
        if self.verbose : print "ListenDB() : "+str

DBManager = ListenDBManager()
