###
#
# Listen is the legal property of mehdi abaakouk <theli48@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###


import gtk
import gobject
import pango
import xml.sax.saxutils

import config
import utils
from song_editor import ListenUISongEditor
from db_manager import DBManager

class ListenUITreeviewSong(gtk.TreeView):
    def __init__(self,media_organizer,allow_column={"title":True,"album":True,"artist":True,"track":True,"duration":True},sortable=True):

        self.media_organizer = media_organizer
        self.player = media_organizer.player
        gtk.TreeView.__init__(self)
        self.allow_column = allow_column


        self.model_song = gtk.ListStore(object,gobject.TYPE_INT,gobject.TYPE_STRING,gobject.TYPE_INT,gobject.TYPE_STRING,gobject.TYPE_STRING)#donnees, chaine de trie

        self.renderer_song = gtk.CellRendererText()
        self.renderer_song.set_property("ellipsize",pango.ELLIPSIZE_END)
        self.renderer_song_artist = gtk.CellRendererText()
        self.renderer_song_artist.set_property("ellipsize",pango.ELLIPSIZE_END)
        self.renderer_song_album = gtk.CellRendererText()
        self.renderer_song_album.set_property("ellipsize",pango.ELLIPSIZE_END)
        self.renderer_song_duration = gtk.CellRendererText()
        self.renderer_song_track = gtk.CellRendererText()


     ### song
        column_song = gtk.TreeViewColumn()
        column_song.set_title(_("Title"))
        column_song.pack_start(self.renderer_song,True)
        column_song.set_expand(True)
        if sortable:
            column_song.set_sort_column_id(2)
        column_song.set_sizing(gtk.TREE_VIEW_COLUMN_GROW_ONLY)
        column_song.set_cell_data_func(self.renderer_song, self.__cell_data_func_song)

        column_song_album = gtk.TreeViewColumn()
        column_song_album.set_title(_("Album"))
        column_song_album.set_min_width(100)
        if sortable:
            column_song_album.set_sort_column_id(5)
        column_song_album.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column_song_album.pack_start(self.renderer_song_album,True)
        column_song_album.set_cell_data_func(self.renderer_song_album, self.__cell_data_func_song)


        column_song_artist = gtk.TreeViewColumn()
        column_song_artist.set_title(_("Artist"))
        if sortable:
            column_song_artist.set_sort_column_id(4)
        column_song_artist.set_min_width(100)
        column_song_artist.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        column_song_artist.pack_start(self.renderer_song_artist,True)
        column_song_artist.set_cell_data_func(self.renderer_song_artist, self.__cell_data_func_song)


        column_song_duration = gtk.TreeViewColumn()
        if sortable:
            column_song_duration.set_sort_column_id(3)
        column_song_duration.set_title(_("Length"))
        column_song_duration.pack_start(self.renderer_song_duration,True)
        column_song_duration.set_cell_data_func(self.renderer_song_duration, self.__cell_data_func_song)

        column_song_track = gtk.TreeViewColumn()
        column_song_track.set_title("#")
        if sortable:
            column_song_track.set_sort_column_id(1)
        column_song_track.set_fixed_width(30)
        column_song_track.pack_start(self.renderer_song_track,True)
        column_song_track.set_cell_data_func(self.renderer_song_track, self.__cell_data_func_song)

        if self.allow_column.has_key("track") and self.allow_column["track"]:
            self.append_column(column_song_track)
        if self.allow_column.has_key("title") and self.allow_column["title"]:
            self.append_column(column_song)
        if self.allow_column.has_key("duration") and self.allow_column["duration"]:
            self.append_column(column_song_duration)
        if self.allow_column.has_key("album") and self.allow_column["album"]:
            self.append_column(column_song_album)
        if self.allow_column.has_key("artist") and self.allow_column["artist"]:
            self.append_column(column_song_artist)

        targets = [("text/uri-list", 0, 0)]
        #self.tree_song.enable_model_drag_dest(targets, gtk.gdk.ACTION_DEFAULT|gtk.gdk.ACTION_COPY)
        self.enable_model_drag_source(
            gtk.gdk.BUTTON1_MASK|gtk.gdk.SHIFT_MASK|gtk.gdk.CONTROL_MASK, targets,
            gtk.gdk.ACTION_COPY)

        self.set_rules_hint(True)

        self.connect("row-activated", self.on_row_song_activated)
        self.connect("drag-data-get", self.on_song_drag_data_get)
        self.connect("button_press_event", self.on_button_press_event)
        self.set_search_equal_func(self.on_song_search)
        self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

        self.menu = self.get_menu()

    """""""""""""""""""""""""""
        FUNC ACTION
    """""""""""""""""""""""""""

    def remove_selected(self,btn=None):
        model, rows = self.get_selection().get_selected_rows()
        if len(rows) < 1:
            return
        rows.reverse()
        for row in rows:
            DBManager.delete_song(model[row][0])
            del model[row]

        #Refresh song cache
        from media_source import ListenMLBrowserSource
        for source in self.media_organizer.list_source:
            if isinstance(source,ListenMLBrowserSource):
                source.widget.refresh_cache()
                source.widget.populate_view()

    def edit_selected(self,btn=None):
        model, rows = self.get_selection().get_selected_rows()
        if len(rows) < 1:
            return
        list_song = list(model.get_value(model.get_iter(row), 0) for row in rows)
        window = ListenUISongEditor(self.get_toplevel(),list_song)

    def play_selected(self,btn=None):
        model, rows = self.get_selection().get_selected_rows()
        if len(rows) < 1:
            return
        play = True
        for iter in rows:
            song = model.get_value(model.get_iter(iter), 0)
            self.player.playlist.add_song(song,False,play)
            play = False
        #print pos

    def enqueue_selected(self,btn=None):
        model, rows = self.get_selection().get_selected_rows()
        if len(rows) < 1:
            return

        for iter in rows:
            song = model.get_value(model.get_iter(iter), 0)
            self.player.playlist.add_song(song)

    def info_artist(self,btn=None):
        model, rows = self.get_selection().get_selected_rows()
        song = model.get_value(model.get_iter(rows[0][0]),0)
        c = self.get_context_widget()
        c.wikipedia_widget.set_action("artist")
        c.wikipedia_widget.media_media_cb(None,song)
        c.notebook.set_current_page(3)
        #FIXME GET POSITION OF CONTEXT WIDGET NOT THE FIRST
        self.media_organizer.populate(0)

    def info_album(self,btn=None):
        model, rows = self.get_selection().get_selected_rows()
        song = model.get_value(model.get_iter(rows[0][0]),0)
        c = self.get_context_widget()
        c.wikipedia_widget.set_action("album")
        c.wikipedia_widget.media_media_cb(None,song)
        c.notebook.set_current_page(3)
        #FIXME GET POSITION OF CONTEXT WIDGET NOT THE FIRST
        self.media_organizer.populate(0)

    def info_lyrics(self,btn=None):
        model, rows = self.get_selection().get_selected_rows()
        song = model.get_value(model.get_iter(rows[0][0]),0)
        c = self.get_context_widget()
        c.lyrics_widget.new_media_cb(None,song)
        c.notebook.set_current_page(2)

        #FIXME GET POSITION OF CONTEXT WIDGET NOT THE FIRST
        self.media_organizer.populate(0)

    def get_context_widget(self):
        from media_source import ListenContextSource
        for source in self.media_organizer.list_source:
            if isinstance(source,ListenContextSource):
                return source.widget

    """""""""""""""""""""""""""
        FUNC SELECTION
    """""""""""""""""""""""""""
    def on_row_song_activated(self, treeview, path, view_column):
        self.play_selected()


    def on_button_press_event(self, widget, event):
        model, rows = self.get_selection().get_selected_rows()

        if event.button == 3 and len(rows)>0:
            self.menu.popup(None,None,None,event.button,event.time)

    """""""""""""""""""""""
        CELL RENDER FUNC
    """""""""""""""""""""""
    def __cell_data_func_song(self, column, cell, model, iter):
        title = column.get_title()
        if title == _("Title"):
            value = model.get_value(iter, 2)
            if value=="":
                value = model.get_value(iter, 0)
                value = "("+value.get_property("uri")+")"
        elif title == _("Artist"):
             value = model.get_value(iter, 4)
        elif title == _("Album")  :
             value = model.get_value(iter, 5)
        elif title == _("Length") :
             value = model.get_value(iter, 3)
             value  = utils.duration_to_string(value)
        elif title == "#" :
             value = model.get_value(iter, 1)
             if value == 0:
                 value = ""
             else:
                 value = "%d"%value
        else:
             value = ""
        value = xml.sax.saxutils.escape(value)
        if config.get("setting","mlib_smallfont")=="true":
            value="<span size=\"small\">"+value+"</span>"
        #print "TITLE: ",title," - VALEUR: ",value
        cell.set_property("markup",value )


    """""""""""""""""""""
        FUNC FILTER
    """""""""""""""""""""
    def on_song_search(self,model, column, key, iterator):
        title = model.get_value(iterator, 0).sprint("title")
        if title.lower().find(key.lower()) == 0:
          return False
        return True


    """""""""""""""""""""
        FUNC DRAG DROP
    """""""""""""""""""""
    def on_song_drag_data_get(self,treeview, context, selection, info, timestamp):
        #print "SONG VIEW : drop data get"
        model, rows = self.get_selection().get_selected_rows()

        list_uri = []
        list_song = []
        for row in rows:
            song = model.get_value(model.get_iter(row[0]),0)
            list_song.append(song)
            uri = song.get_property("uri")
            uri = utils.convert_to_uri(uri)
            list_uri.append(uri)

        self.get_toplevel().last_drag_drop = list_song
        selection.set_uris(list_uri)


    def get_menu(self):

        menu_ui = """
            <ui>
                <popup name="SongMenu">
                  <menuitem action="Play"/>
                  <menuitem action="Queue"/>
                  <menuitem action="Edit"/>
                  <menuitem action="Delete"/>
                  <separator/>
                  <menuitem action="Lyrics"/>
                  <menuitem action="Artist"/>
                  <menuitem action="Album"/>
                </popup>
            </ui>
            """

        self.uimanager = gtk.UIManager()
        self.uimanager.add_ui_from_string(menu_ui)
        accelgroup = self.uimanager.get_accel_group()

        actiongroup = gtk.ActionGroup('Listen')
        actiongroup.add_actions([('Play', gtk.STOCK_MEDIA_PLAY, _('_Play'), None,
                                         _('Play this song'), self.play_selected)])
        actiongroup.add_actions([('Queue', gtk.STOCK_ADD, _('_Queue'), None,
                                         _('Queue this song'), self.enqueue_selected)])
        actiongroup.add_actions([('Edit', gtk.STOCK_EDIT, _('_Edit'), None,
                                         _('Edit this song'), self.edit_selected)])
        actiongroup.add_actions([('Delete', gtk.STOCK_EDIT, _('_Remove from library'), None,
                                         _('Remove this song from the library'), self.remove_selected)])



        actiongroup.add_actions([('Artist', gtk.STOCK_ABOUT, _('About _artist'), None,
                                         _('More information about artist'), self.info_artist)])
        actiongroup.add_actions([('Album', gtk.STOCK_ABOUT, _('About a_lbum'), None,
                                         _('More information about album'), self.info_album)])
        actiongroup.add_actions([('Lyrics', gtk.STOCK_EDIT, _('See _Lyrics'), None,
                                         _('See lyrics'), self.info_lyrics)])




        self.uimanager.insert_action_group(actiongroup, 1)
        #gobject.idle_add(self.get_toplevel().add_accel_group,accelgroup)

        return self.uimanager.get_widget("/SongMenu")

